import { NgZone } from "@angular/core";
import { Observable, Operator, Subscriber, TeardownLogic } from "rxjs";
export declare class CacheService {
    private ngZone;
    private platformId;
    changes: Observable<CacheEvent>;
    private $store;
    private $changes;
    private $expirations;
    private $timer;
    constructor(ngZone: NgZone, platformId: any);
    has(key: string): boolean;
    get(key: string): CacheValueObservable<any>;
    set(key: string, value: any, expires?: number | string): this;
    delete(key: string): this;
    serialize(): string;
    deserialize(json: string): void;
    /**
     * @returns number of milliseconds elapsed since 1 January 1970 00:00:00 UTC
     */
    private normalizeExpires;
    private addToExpirationQueue;
    private triggerExpirationTimer;
    private removeExpiredEntries;
    private setTimer;
    private clearTimer;
}
export declare type CacheEvent = CacheSetEvent | CacheDeleteEvent | CacheExpireEvent;
export interface CacheSetEvent {
    type: "set";
    key: string;
    value: any;
    expires: number;
}
export interface CacheDeleteEvent {
    type: "delete";
    key: string;
    value: any;
    expires: number;
}
export interface CacheExpireEvent {
    type: "expire";
    key: string;
    value: any;
    expires: number;
}
export declare class CacheValueObservable<T> extends Observable<T> {
    private cache;
    private key;
    static create<T>(cache: CacheService, key: string): CacheValueObservable<T>;
    constructor(cache: CacheService, key: string);
    miss(fetch: FetchFunction<T>): CacheValueObservable<T>;
    expires(callback: ExpiresCallback<T>): CacheValueObservable<T>;
    current(): T | undefined;
}
export declare class CacheValueMissOperator<T> implements Operator<T, T> {
    private cache;
    private key;
    private fetch;
    constructor(cache: CacheService, key: string, fetch: FetchFunction<T>);
    call(subscriber: Subscriber<T>, source: any): TeardownLogic;
}
export declare type FetchFunction<T> = () => FetchResult<T>;
export declare type FetchResult<T> = FetchResultValue<T> | FetchResultValueWithOptions<T>;
export declare type FetchResultValue<T> = T | Promise<T> | Observable<T>;
export interface FetchResultValueWithOptions<T> {
    value: FetchResultValue<T>;
    expires?: number | ((val: any) => number);
}
export declare type ExpiresCallback<T> = (value: T) => ExpiresCallbackResult<T>;
export declare type ExpiresCallbackResult<T> = T | Promise<T> | Observable<T>;
