"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@angular/common");
const core_1 = require("@angular/core");
const rxjs_1 = require("rxjs");
const rxjs_2 = require("rxjs");
const operators_1 = require("rxjs/operators");
let CacheService = class CacheService {
    constructor(ngZone, platformId) {
        this.ngZone = ngZone;
        this.platformId = platformId;
        this.$store = {};
        this.$changes = new rxjs_1.Subject();
        this.$expirations = [];
        this.changes = this.$changes.asObservable().pipe(operators_1.share());
    }
    has(key) {
        return this.$store.hasOwnProperty(key);
    }
    get(key) {
        return CacheValueObservable.create(this, key);
    }
    set(key, value, expires) {
        expires = this.normalizeExpires(expires);
        this.$store[key] = { value, expires };
        this.addToExpirationQueue(key, expires);
        this.$changes.next({ type: "set", key, value, expires });
        return this;
    }
    delete(key) {
        const entry = this.$store[key];
        if (entry) {
            delete this.$store[key];
            const { value, expires } = entry;
            this.$changes.next({ type: "delete", key, value, expires });
        }
        return this;
    }
    serialize() {
        return JSON.stringify(this.$store);
    }
    deserialize(json) {
        const data = JSON.parse(json);
        Object.keys(data).forEach((key) => {
            const { value, expires } = data[key];
            this.set(key, value, expires);
        });
    }
    /**
     * @returns number of milliseconds elapsed since 1 January 1970 00:00:00 UTC
     */
    normalizeExpires(expires) {
        if (typeof expires === "number") {
            if (expires < 0) {
                // never expires
                return -1;
            }
            else if (expires < 365 * 24 * 60 * 60 * 1000) {
                // treat small numbers (less than 1 year) as relative time
                return Date.now() + expires;
            }
            else {
                // treat big numbers as absolute time
                return expires;
            }
        }
        else if (typeof expires === "string") {
            if (!/^\d+(ms|s|m|h|d)?$/.test(expires)) {
                throw new Error(`invalid expires value "${expires}"`);
            }
            const numeral = parseInt(expires.slice(0, -1));
            const unit = expires.slice(-1);
            switch (unit) {
                case "ms":
                    return Date.now() + numeral;
                case "s":
                    return Date.now() + numeral * 1000;
                case "m":
                    return Date.now() + numeral * 60 * 1000;
                case "h":
                    return Date.now() + numeral * 60 * 60 * 1000;
                case "d":
                    return Date.now() + numeral * 24 * 60 * 60 * 1000;
                default:
                    // shoult not hit here
                    throw new Error("unknown error");
            }
        }
        else {
            return -1;
        }
    }
    addToExpirationQueue(key, expires) {
        // no need to check expiration on server side
        if (common_1.isPlatformServer(this.platformId))
            return;
        // if the key is already in queue, remove it
        const duplicateIndex = this.$expirations.findIndex((item) => item.key === key);
        if (duplicateIndex >= 0)
            this.$expirations.splice(duplicateIndex, 1);
        if (duplicateIndex === 0 && this.$timer)
            this.clearTimer();
        // if expires is set, insert it to the queue ordered by expires
        if (expires > 0) {
            let insertIndex = this.$expirations.findIndex((item) => item.expires > expires);
            if (insertIndex < 0)
                insertIndex = this.$expirations.length;
            this.$expirations.splice(insertIndex, 0, { key, expires });
            if (insertIndex === 0 && this.$timer)
                this.clearTimer();
        }
        // trigger timer if needed
        if (this.$expirations.length > 0 && !this.$timer)
            this.triggerExpirationTimer();
    }
    triggerExpirationTimer() {
        if (this.$expirations.length > 0 && !this.$timer) {
            const delay = this.$expirations[0].expires - Date.now();
            this.setTimer(delay, () => {
                this.removeExpiredEntries();
                this.triggerExpirationTimer();
            });
        }
    }
    removeExpiredEntries() {
        while (this.$expirations.length > 0) {
            const { key, expires } = this.$expirations[0];
            if (Date.now() < expires)
                break;
            this.$expirations.shift();
            const value = this.$store[key].value;
            delete this.$store[key];
            this.$changes.next({ type: "expire", key, value, expires });
        }
    }
    setTimer(delay, task) {
        // run timer outside of angular zone so that it won't block protractor
        this.ngZone.runOutsideAngular(() => {
            this.$timer = setTimeout(() => {
                this.$timer = undefined;
                this.ngZone.run(task);
            }, delay);
        });
    }
    clearTimer() {
        if (this.$timer)
            clearTimeout(this.$timer);
        this.$timer = undefined;
    }
};
CacheService = __decorate([
    core_1.Injectable(),
    __param(1, core_1.Inject(core_1.PLATFORM_ID)),
    __metadata("design:paramtypes", [core_1.NgZone, Object])
], CacheService);
exports.CacheService = CacheService;
////////////
class CacheValueObservable extends rxjs_1.Observable {
    constructor(cache, key) {
        super();
        this.cache = cache;
        this.key = key;
    }
    static create(cache, key) {
        const observable = new CacheValueObservable(cache, key);
        const changes = cache.changes.pipe(operators_1.filter((event) => event.type === "set" && event.key === key), operators_1.map((event) => event.value));
        if (cache.has(key)) {
            const initialValue = cache.$store[key].value;
            observable.source = changes.pipe(operators_1.startWith(initialValue));
        }
        else {
            observable.source = changes;
        }
        return observable;
    }
    miss(fetch) {
        const observable = new CacheValueObservable(this.cache, this.key);
        observable.source = this;
        observable.operator = new CacheValueMissOperator(this.cache, this.key, fetch);
        return observable;
    }
    expires(callback) {
        const observable = new CacheValueObservable(this.cache, this.key);
        observable.source = this.cache.changes.pipe(operators_1.filter((event) => event.type === "expire" && event.key === this.key), operators_1.map((event) => callback(event.value)), operators_1.flatMap((value) => {
            if (isObservable(value)) {
                return value.pipe(operators_1.first());
            }
            else if (isPromise(value)) {
                return rxjs_2.from(value);
            }
            else {
                return rxjs_2.of(value);
            }
        }), operators_1.merge(this));
        return observable;
    }
    current() {
        if (this.cache.has(this.key)) {
            return this.cache.$store[this.key].value;
        }
        else {
            return undefined;
        }
    }
}
exports.CacheValueObservable = CacheValueObservable;
class CacheValueMissOperator {
    constructor(cache, key, fetch) {
        this.cache = cache;
        this.key = key;
        this.fetch = fetch;
    }
    call(subscriber, source) {
        const cache = this.cache;
        const key = this.key;
        const fetch = this.fetch;
        const fetchingKey = `CacheService:Fetching:${key}`;
        if (!this.cache.has(this.key) && !this.cache.has(fetchingKey)) {
            this.cache.set(fetchingKey, true);
            let expires;
            const onResolve = (value) => {
                // this must be async for subscribers to receive this value
                Promise.resolve().then(() => {
                    if (typeof expires === "function") {
                        expires = expires(value);
                    }
                    this.cache.delete(fetchingKey);
                    this.cache.set(key, value, expires);
                });
            };
            const onError = (err) => {
                this.cache.delete(fetchingKey);
                subscriber.error(err);
            };
            let fetchResult;
            try {
                fetchResult = fetch();
            }
            catch (err) {
                return onError(err);
            }
            let fetchResultValue;
            if (isFetchResultValueWithOptions(fetchResult)) {
                fetchResultValue = fetchResult.value;
                expires = fetchResult.expires;
            }
            else {
                fetchResultValue = fetchResult;
                expires = -1;
            }
            if (isObservable(fetchResultValue)) {
                fetchResultValue.pipe(operators_1.first()).subscribe(onResolve, onError);
            }
            else if (isPromise(fetchResultValue)) {
                fetchResultValue.then(onResolve, onError);
            }
            else {
                onResolve(fetchResultValue);
            }
        }
        return source._subscribe(subscriber);
    }
}
exports.CacheValueMissOperator = CacheValueMissOperator;
////////////
function isFetchResultValueWithOptions(obj) {
    return obj && obj.hasOwnProperty("value");
}
function isObservable(obj) {
    return obj && obj.subscribe;
}
function isPromise(obj) {
    return obj && obj.then;
}
//# sourceMappingURL=cache.service.js.map